'use strict';

var EventEmitter = require('events').EventEmitter,
    objectAssign = require("object-assign"),
    path = require("path"),
    mkdirp = require("mkdirp"),
    sanitize = require("sanitize-filename"),
    request = require("request"),
    fs = require("fs"),
    urlUtil = require("url");

var SaveUrl = function SaveUrl() {
    this._defualtOptions = {
        baseDiskPath: __dirname,
        sanatizeFileName: true,
        fileName: "index.html",
        includeHtml: true,
        includeDomain: true,
        userAgent: "save-url (https://github.com/ryanwild/save-url.git)"
    };

    this._setOptions(this._defualtOptions);
};

objectAssign(SaveUrl.prototype, EventEmitter.prototype);
module.exports = SaveUrl;

SaveUrl.prototype._setOptions = function (options) {
    this.options = objectAssign(this._defualtOptions, options);
};

/**
 * Returns the target disk path from url
 *
 * @return {string}
 */
SaveUrl.prototype._getPathFromUrl = function(url) {
    var urlObject = urlUtil.parse(url);
    var filePath;

    if(this.options.includeDomain) {
      filePath = urlObject.hostname + urlObject.pathname;
    } else {
        filePath = urlObject.pathname;
    }

    filePath = filePath.replace("/", path.sep);

    if(this.options.includeDomain === false && filePath === path.sep) {
        filePath = path.resolve(this.options.baseDiskPath);
    } else {
        filePath = path.resolve(path.join(this.options.baseDiskPath, filePath));
    }

    return filePath;
};

SaveUrl.prototype.save = function(url, options, done) {
    this._setOptions(options);

    var outputPath = this._getPathFromUrl(url);
    mkdirp(outputPath);

    if(this.options.includeHtml) {
        if(this.options.sanatizeFileName) {
            this.options.fileName = sanitize(this.options.fileName);
        }
        outputPath = path.join(outputPath, this.options.fileName);

        request.get(
            url,
            {
                headers : {
                    'User-Agent': this.options.userAgent
                }
            },
            function(err, httpResponse, body) {
                if(!err) {
                    fs.writeFile(outputPath, body, function(fileErr) {
                        done(fileErr, httpResponse, body);
                    });
                } else {
                    done(err, httpResponse, body);
                }
            }
        );
    }
};